from __future__ import annotations

import sys
from typing import TYPE_CHECKING

import pytest

from python_discovery import PythonSpec
from python_discovery._py_info import normalize_isa
from python_discovery._specifier import SimpleSpecifierSet as SpecifierSet

if TYPE_CHECKING:
    from pathlib import Path


def test_bad_py_spec() -> None:
    text = "python2.3.4.5"
    spec = PythonSpec.from_string_spec(text)
    assert text in repr(spec)
    assert spec.str_spec == text
    assert spec.path == text
    content = vars(spec)
    del content["str_spec"]
    del content["path"]
    assert all(v is None for v in content.values())


def test_py_spec_first_digit_only_major() -> None:
    spec = PythonSpec.from_string_spec("278")
    assert spec.major == 2
    assert spec.minor == 78


def test_spec_satisfies_path_ok() -> None:
    spec = PythonSpec.from_string_spec(sys.executable)
    assert spec.satisfies(spec) is True


def test_spec_satisfies_path_nok(tmp_path: pytest.TempPathFactory) -> None:
    spec = PythonSpec.from_string_spec(sys.executable)
    of = PythonSpec.from_string_spec(str(tmp_path))
    assert spec.satisfies(of) is False


def test_spec_satisfies_arch() -> None:
    spec_1 = PythonSpec.from_string_spec("python-32")
    spec_2 = PythonSpec.from_string_spec("python-64")

    assert spec_1.satisfies(spec_1) is True
    assert spec_2.satisfies(spec_1) is False


def test_spec_satisfies_free_threaded() -> None:
    spec_1 = PythonSpec.from_string_spec("python3.13t")
    spec_2 = PythonSpec.from_string_spec("python3.13")

    assert spec_1.satisfies(spec_1) is True
    assert spec_1.free_threaded is True
    assert spec_2.satisfies(spec_1) is False
    assert spec_2.free_threaded is False


@pytest.mark.parametrize(
    ("req", "spec"),
    [("py", "python"), ("jython", "jython"), ("CPython", "cpython")],
)
def test_spec_satisfies_implementation_ok(req: str, spec: str) -> None:
    spec_1 = PythonSpec.from_string_spec(req)
    spec_2 = PythonSpec.from_string_spec(spec)
    assert spec_1.satisfies(spec_1) is True
    assert spec_2.satisfies(spec_1) is True


def test_spec_satisfies_implementation_nok() -> None:
    spec_1 = PythonSpec.from_string_spec("cpython")
    spec_2 = PythonSpec.from_string_spec("jython")
    assert spec_2.satisfies(spec_1) is False
    assert spec_1.satisfies(spec_2) is False


def _version_satisfies_pairs() -> list[tuple[str, str]]:
    target: set[tuple[str, str]] = set()
    version = tuple(str(i) for i in sys.version_info[0:3])
    for threading in (False, True):
        for depth in range(len(version) + 1):
            req = ".".join(version[0:depth])
            for sub in range(depth + 1):
                sat = ".".join(version[0:sub])
                if sat:
                    target.add((req, sat))
                target.add((sat, req))
                if threading and sat and req:
                    target.add((f"{req}t", f"{sat}t"))
                    target.add((f"{sat}t", f"{req}t"))

    return sorted(target)


@pytest.mark.parametrize(("req", "spec"), _version_satisfies_pairs())
def test_version_satisfies_ok(req: str, spec: str) -> None:
    req_spec = PythonSpec.from_string_spec(f"python{req}")
    sat_spec = PythonSpec.from_string_spec(f"python{spec}")
    assert sat_spec.satisfies(req_spec) is True


def _version_not_satisfies_pairs() -> list[tuple[str, str]]:
    target: set[tuple[str, str]] = set()
    version = tuple(str(i) for i in sys.version_info[0:3])
    for major in range(len(version)):
        req = ".".join(version[0 : major + 1])
        for minor in range(major + 1):
            sat_ver: list[int] = [int(v) for v in sys.version_info[0 : minor + 1]]
            for patch in range(minor + 1):
                for offset in [1, -1]:
                    temp = sat_ver.copy()
                    temp[patch] += offset
                    if temp[patch] < 0:
                        continue  # pragma: no cover
                    sat = ".".join(str(i) for i in temp)
                    target.add((req, sat))
    return sorted(target)


@pytest.mark.parametrize(("req", "spec"), _version_not_satisfies_pairs())
def test_version_satisfies_nok(req: str, spec: str) -> None:
    req_spec = PythonSpec.from_string_spec(f"python{req}")
    sat_spec = PythonSpec.from_string_spec(f"python{spec}")
    assert sat_spec.satisfies(req_spec) is False


def test_relative_spec(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.chdir(tmp_path)
    a_relative_path = str((tmp_path / "a" / "b").relative_to(tmp_path))
    spec = PythonSpec.from_string_spec(a_relative_path)
    assert spec.path == a_relative_path


@pytest.mark.parametrize(
    ("text", "expected"),
    [
        (">=3.12", ">=3.12"),
        ("python>=3.12", ">=3.12"),
        ("cpython!=3.11.*", "!=3.11.*"),
        ("<=3.13,>=3.12", "<=3.13,>=3.12"),
    ],
)
def test_specifier_parsing(text: str, expected: str) -> None:
    spec = PythonSpec.from_string_spec(text)
    assert spec.version_specifier == SpecifierSet.from_string(expected)


def test_specifier_with_implementation() -> None:
    spec = PythonSpec.from_string_spec("cpython>=3.12")
    assert spec.implementation == "cpython"
    assert spec.version_specifier == SpecifierSet.from_string(">=3.12")


def test_specifier_satisfies_with_partial_information() -> None:
    spec = PythonSpec.from_string_spec(">=3.12")
    candidate = PythonSpec.from_string_spec("python3.12")
    assert candidate.satisfies(spec) is True


@pytest.mark.parametrize(
    ("spec_str", "expected_machine"),
    [
        pytest.param("cpython3.12-64-arm64", "arm64", id="arm64"),
        pytest.param("cpython3.12-64-x86_64", "x86_64", id="x86_64"),
        pytest.param("cpython3.12-32-x86", "x86", id="x86"),
        pytest.param("cpython3.12-64-aarch64", "arm64", id="aarch64"),
        pytest.param("cpython3.12-64-ppc64le", "ppc64le", id="ppc64le"),
        pytest.param("cpython3.12-64-s390x", "s390x", id="s390x"),
        pytest.param("cpython3.12-64-riscv64", "riscv64", id="riscv64"),
        pytest.param("cpython3.12-64", None, id="no-machine"),
        pytest.param("cpython3.12", None, id="no-arch-no-machine"),
        pytest.param("python3.12-64-arm64", "arm64", id="python-impl"),
    ],
)
def test_spec_parse_machine(spec_str: str, expected_machine: str | None) -> None:
    spec = PythonSpec.from_string_spec(spec_str)
    assert spec.machine == expected_machine


@pytest.mark.parametrize(
    ("spec_str", "expected_arch", "expected_machine"),
    [
        pytest.param("cpython3.12-64-arm64", 64, "arm64", id="64bit-arm64"),
        pytest.param("cpython3.12-32-x86", 32, "x86", id="32bit-x86"),
        pytest.param("cpython3.12-64", 64, None, id="64bit-no-machine"),
    ],
)
def test_spec_parse_arch_and_machine_together(spec_str: str, expected_arch: int, expected_machine: str | None) -> None:
    spec = PythonSpec.from_string_spec(spec_str)
    assert spec.architecture == expected_arch
    assert spec.machine == expected_machine


@pytest.mark.parametrize(
    ("left", "right", "expected"),
    [
        pytest.param("cpython3.12-64-arm64", "cpython3.12-64-arm64", True, id="same-machine"),
        pytest.param("cpython3.12-64-arm64", "cpython3.12-64-x86_64", False, id="different-machine"),
        pytest.param("cpython3.12-64-arm64", "cpython3.12-64", True, id="none-matches-any"),
        pytest.param("cpython3.12-64-amd64", "cpython3.12-64-x86_64", True, id="amd64-eq-x86_64"),
        pytest.param("cpython3.12-64-aarch64", "cpython3.12-64-arm64", True, id="aarch64-eq-arm64"),
    ],
)
def test_spec_satisfies_machine(left: str, right: str, expected: bool) -> None:
    assert PythonSpec.from_string_spec(left).satisfies(PythonSpec.from_string_spec(right)) is expected


@pytest.mark.parametrize(
    ("isa", "normalized"),
    [
        pytest.param("amd64", "x86_64", id="amd64"),
        pytest.param("aarch64", "arm64", id="aarch64"),
        pytest.param("x86_64", "x86_64", id="x86_64"),
        pytest.param("arm64", "arm64", id="arm64"),
        pytest.param("x86", "x86", id="x86"),
        pytest.param("i386", "x86", id="i386"),
        pytest.param("i486", "x86", id="i486"),
        pytest.param("i586", "x86", id="i586"),
        pytest.param("i686", "x86", id="i686"),
        pytest.param("ppc64le", "ppc64le", id="ppc64le"),
        pytest.param("powerpc", "ppc", id="powerpc"),
        pytest.param("powerpc64", "ppc64", id="powerpc64"),
        pytest.param("powerpc64le", "ppc64le", id="powerpc64le"),
        pytest.param("riscv64", "riscv64", id="riscv64"),
        pytest.param("s390x", "s390x", id="s390x"),
        pytest.param("sparcv9", "sparc64", id="sparcv9"),
        pytest.param("sparc64", "sparc64", id="sparc64"),
        pytest.param("alpha", "alpha", id="alpha-passthrough"),
    ],
)
def test_normalize_isa(isa: str, normalized: str) -> None:
    assert normalize_isa(isa) == normalized


@pytest.mark.parametrize(
    ("spec_str", "in_repr"),
    [
        pytest.param("cpython3.12-64-arm64", "machine=arm64", id="with-machine"),
        pytest.param("cpython3.12-64", "architecture=64", id="without-machine"),
    ],
)
def test_spec_repr_machine(spec_str: str, in_repr: str) -> None:
    assert in_repr in repr(PythonSpec.from_string_spec(spec_str))



"""Self-contained Python interpreter discovery."""

from __future__ import annotations

from importlib.metadata import version

from ._cache import ContentStore, DiskCache, PyInfoCache
from ._discovery import get_interpreter
from ._py_info import KNOWN_ARCHITECTURES, PythonInfo, normalize_isa
from ._py_spec import PythonSpec
from ._specifier import SimpleSpecifier, SimpleSpecifierSet, SimpleVersion

__version__ = version("python-discovery")

__all__ = [
    "KNOWN_ARCHITECTURES",
    "ContentStore",
    "DiskCache",
    "PyInfoCache",
    "PythonInfo",
    "PythonSpec",
    "SimpleSpecifier",
    "SimpleSpecifierSet",
    "SimpleVersion",
    "__version__",
    "get_interpreter",
    "normalize_isa",
]



from __future__ import annotations

import sys
import textwrap

import pytest


@pytest.mark.usefixtures("_mock_registry")
def test_pep514_discovers_interpreters() -> None:
    from python_discovery._windows._pep514 import discover_pythons

    interpreters = list(discover_pythons())
    assert len(interpreters) == 12
    companies = {i[0] for i in interpreters}
    assert "ContinuumAnalytics" in companies
    assert "PythonCore" in companies
    assert "CompanyA" in companies


@pytest.mark.usefixtures("_mock_registry")
def test_pep514_parse_functions() -> None:
    from python_discovery._windows._pep514 import parse_arch, parse_version

    assert parse_arch("64bit") == 64
    assert parse_arch("32bit") == 32
    with pytest.raises(ValueError, match="invalid format"):
        parse_arch("magic")
    with pytest.raises(ValueError, match="arch is not string"):
        parse_arch(100)

    assert parse_version("3.12") == (3, 12, None)
    assert parse_version("3.12.1") == (3, 12, 1)
    assert parse_version("3") == (3, None, None)
    with pytest.raises(ValueError, match="invalid format"):
        parse_version("3.X")
    with pytest.raises(ValueError, match="version is not string"):
        parse_version(2778)


@pytest.mark.skipif(sys.platform != "win32", reason="path joining differs on POSIX")
@pytest.mark.usefixtures("_mock_registry")
def test_pep514() -> None:
    from python_discovery._windows._pep514 import discover_pythons

    interpreters = list(discover_pythons())
    assert interpreters == [
        ("ContinuumAnalytics", 3, 10, 32, False, "C:\\Users\\user\\Miniconda3\\python.exe", None),
        ("ContinuumAnalytics", 3, 10, 64, False, "C:\\Users\\user\\Miniconda3-64\\python.exe", None),
        (
            "PythonCore",
            3,
            9,
            64,
            False,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python39\\python.exe",
            None,
        ),
        (
            "PythonCore",
            3,
            9,
            64,
            False,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python39\\python.exe",
            None,
        ),
        (
            "PythonCore",
            3,
            8,
            64,
            False,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python38\\python.exe",
            None,
        ),
        (
            "PythonCore",
            3,
            9,
            64,
            False,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python39\\python.exe",
            None,
        ),
        (
            "PythonCore",
            3,
            10,
            32,
            False,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python310-32\\python.exe",
            None,
        ),
        (
            "PythonCore",
            3,
            12,
            64,
            False,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python312\\python.exe",
            None,
        ),
        (
            "PythonCore",
            3,
            13,
            64,
            True,
            "C:\\Users\\user\\AppData\\Local\\Programs\\Python\\Python313\\python3.13t.exe",
            None,
        ),
        ("CompanyA", 3, 6, 64, False, "Z:\\CompanyA\\Python\\3.6\\python.exe", None),
        ("PythonCore", 2, 7, 64, False, "C:\\Python27\\python.exe", None),
        ("PythonCore", 3, 7, 64, False, "C:\\Python37\\python.exe", None),
    ]


@pytest.mark.skipif(sys.platform != "win32", reason="path joining differs on POSIX")
@pytest.mark.usefixtures("_mock_registry")
def test_pep514_run(capsys: pytest.CaptureFixture[str], caplog: pytest.LogCaptureFixture) -> None:
    from python_discovery._windows import _pep514 as pep514

    pep514._run()
    out, err = capsys.readouterr()
    py = r"C:\Users\user\AppData\Local\Programs\Python"
    expected = textwrap.dedent(
        rf"""
    ('CompanyA', 3, 6, 64, False, 'Z:\\CompanyA\\Python\\3.6\\python.exe', None)
    ('ContinuumAnalytics', 3, 10, 32, False, 'C:\\Users\\user\\Miniconda3\\python.exe', None)
    ('ContinuumAnalytics', 3, 10, 64, False, 'C:\\Users\\user\\Miniconda3-64\\python.exe', None)
    ('PythonCore', 2, 7, 64, False, 'C:\\Python27\\python.exe', None)
    ('PythonCore', 3, 10, 32, False, '{py}\\Python310-32\\python.exe', None)
    ('PythonCore', 3, 12, 64, False, '{py}\\Python312\\python.exe', None)
    ('PythonCore', 3, 13, 64, True, '{py}\\Python313\\python3.13t.exe', None)
    ('PythonCore', 3, 7, 64, False, 'C:\\Python37\\python.exe', None)
    ('PythonCore', 3, 8, 64, False, '{py}\\Python38\\python.exe', None)
    ('PythonCore', 3, 9, 64, False, '{py}\\Python39\\python.exe', None)
    ('PythonCore', 3, 9, 64, False, '{py}\\Python39\\python.exe', None)
    ('PythonCore', 3, 9, 64, False, '{py}\\Python39\\python.exe', None)
    """,
    ).strip()
    assert out.strip() == expected
    assert not err
    prefix = "PEP-514 violation in Windows Registry at "
    expected_logs = [
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.1/SysArchitecture error: invalid format magic",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.2/SysArchitecture error: arch is not string: 100",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.3 error: no ExecutablePath or default for it",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.3 error: could not load exe with value None",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.11/InstallPath error: missing",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.12/SysVersion error: invalid format magic",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.X/SysVersion error: version is not string: 2778",
        f"{prefix}HKEY_CURRENT_USER/PythonCore/3.X error: invalid format 3.X",
    ]
    assert caplog.messages == expected_logs



from __future__ import annotations

import pytest

from python_discovery._specifier import SimpleSpecifier, SimpleSpecifierSet, SimpleVersion

# --- SimpleVersion ---


@pytest.mark.parametrize(
    ("version_str", "release", "pre_type", "pre_num"),
    [
        pytest.param("3.11.2", (3, 11, 2), None, None, id="basic"),
        pytest.param("3.14.0a1", (3, 14, 0), "a", 1, id="alpha"),
        pytest.param("3.14.0b2", (3, 14, 0), "b", 2, id="beta"),
        pytest.param("3.14.0rc1", (3, 14, 0), "rc", 1, id="rc"),
        pytest.param("3", (3, 0, 0), None, None, id="major-only"),
        pytest.param("3.12", (3, 12, 0), None, None, id="major-minor"),
    ],
)
def test_version_parse(
    version_str: str,
    release: tuple[int, int, int],
    pre_type: str | None,
    pre_num: int | None,
) -> None:
    version = SimpleVersion.from_string(version_str)
    assert version.release == release
    assert version.pre_type == pre_type
    assert version.pre_num == pre_num


def test_version_invalid_raises() -> None:
    with pytest.raises(ValueError, match="Invalid version"):
        SimpleVersion.from_string("not_a_version")


def test_version_eq_same() -> None:
    assert SimpleVersion.from_string("3.11") == SimpleVersion.from_string("3.11")


def test_version_eq_different() -> None:
    assert SimpleVersion.from_string("3.11") != SimpleVersion.from_string("3.12")


def test_version_eq_not_implemented() -> None:
    result = SimpleVersion.from_string("3.11").__eq__("3.11")  # noqa: PLC2801
    assert result is NotImplemented


def test_version_hash() -> None:
    assert hash(SimpleVersion.from_string("3.11")) == hash(SimpleVersion.from_string("3.11"))


@pytest.mark.parametrize(
    ("left", "right", "expected"),
    [
        pytest.param("3.11", "3.12", True, id="release"),
        pytest.param("3.14.0a1", "3.14.0", True, id="prerelease-vs-final"),
        pytest.param("3.14.0", "3.14.0a1", False, id="final-not-less-than-prerelease"),
        pytest.param("3.14.0", "3.14.0", False, id="equal"),
        pytest.param("3.14.0a1", "3.14.0b1", True, id="alpha-lt-beta"),
        pytest.param("3.14.0b1", "3.14.0rc1", True, id="beta-lt-rc"),
        pytest.param("3.14.0a1", "3.14.0a2", True, id="same-type-ordering"),
    ],
)
def test_version_lt(left: str, right: str, expected: bool) -> None:
    assert (SimpleVersion.from_string(left) < SimpleVersion.from_string(right)) is expected


def test_version_lt_not_implemented() -> None:
    result = SimpleVersion.from_string("3.11").__lt__("3.12")  # noqa: PLC2801
    assert result is NotImplemented


def test_version_le() -> None:
    assert SimpleVersion.from_string("3.11") <= SimpleVersion.from_string("3.12")
    assert SimpleVersion.from_string("3.11") <= SimpleVersion.from_string("3.11")


def test_version_gt() -> None:
    assert SimpleVersion.from_string("3.12") > SimpleVersion.from_string("3.11")


def test_version_gt_not_implemented() -> None:
    result = SimpleVersion.from_string("3.11").__gt__("3.11")  # noqa: PLC2801
    assert result is NotImplemented


def test_version_ge() -> None:
    assert SimpleVersion.from_string("3.12") >= SimpleVersion.from_string("3.11")
    assert SimpleVersion.from_string("3.12") >= SimpleVersion.from_string("3.12")


def test_version_str() -> None:
    assert str(SimpleVersion.from_string("3.11")) == "3.11"


def test_version_repr() -> None:
    assert repr(SimpleVersion.from_string("3.11")) == "SimpleVersion('3.11')"


# --- SimpleSpecifier ---


def test_specifier_invalid_raises() -> None:
    with pytest.raises(ValueError, match="Invalid specifier"):
        SimpleSpecifier.from_string("no_operator")


def test_specifier_parse_gte() -> None:
    spec = SimpleSpecifier.from_string(">=3.12")
    assert spec.operator == ">="
    assert spec.version == SimpleVersion.from_string("3.12")
    assert spec.is_wildcard is False


def test_specifier_parse_wildcard() -> None:
    spec = SimpleSpecifier.from_string("==3.11.*")
    assert spec.is_wildcard is True
    assert spec.wildcard_precision == 2


@pytest.mark.parametrize(
    ("spec_str", "version_str", "expected"),
    [
        pytest.param("==3.11.*", "3.11.5", True, id="wildcard-eq-match"),
        pytest.param("==3.11.*", "3.12.0", False, id="wildcard-eq-no-match"),
        pytest.param("!=3.11.*", "3.11.5", False, id="wildcard-ne-match"),
        pytest.param("!=3.11.*", "3.12.0", True, id="wildcard-ne-no-match"),
        pytest.param(">=3.11.*", "3.11.5", False, id="wildcard-unsupported-op"),
        pytest.param(">=3.12", "3.12.0", True, id="gte-match"),
        pytest.param(">=3.12", "3.13.0", True, id="gte-above"),
        pytest.param(">=3.12", "3.11.0", False, id="gte-below"),
        pytest.param("<=3.12", "3.12.0", True, id="lte-match"),
        pytest.param("<=3.12", "3.11.0", True, id="lte-below"),
        pytest.param("<=3.12", "3.13.0", False, id="lte-above"),
        pytest.param(">3.12", "3.13.0", True, id="gt-above"),
        pytest.param(">3.12", "3.12.0", False, id="gt-equal"),
        pytest.param("<3.12", "3.11.0", True, id="lt-below"),
        pytest.param("<3.12", "3.12.0", False, id="lt-equal"),
        pytest.param("==3.12.0", "3.12.0", True, id="eq-match"),
        pytest.param("==3.12.0", "3.12.1", False, id="eq-no-match"),
        pytest.param("!=3.12.0", "3.12.0", False, id="ne-match"),
        pytest.param("!=3.12.0", "3.12.1", True, id="ne-no-match"),
        pytest.param("===3.12", "3.12", True, id="exact-match"),
        pytest.param("===3.12", "3.12.0", False, id="exact-no-match"),
        pytest.param("~=3.12.0", "3.12.5", True, id="compatible-above"),
        pytest.param("~=3.12.0", "3.13.0", False, id="compatible-next-minor"),
        pytest.param("~=3.12.0", "3.11.0", False, id="compatible-below"),
        pytest.param("~=3.12.0", "3.11.9", False, id="compatible-just-below"),
        pytest.param(">=3.12", "not_a_version", False, id="invalid-version"),
    ],
)
def test_specifier_contains(spec_str: str, version_str: str, expected: bool) -> None:
    spec = SimpleSpecifier.from_string(spec_str)
    assert spec.contains(version_str) is expected


def test_specifier_contains_version_none() -> None:
    spec = SimpleSpecifier(
        spec_str=">=3.12",
        operator=">=",
        version_str="3.12",
        is_wildcard=False,
        wildcard_precision=None,
        version=None,
    )
    assert spec.contains("3.12") is False


def test_specifier_wildcard_version_none() -> None:
    spec = SimpleSpecifier(
        spec_str="==3.11.*",
        operator="==",
        version_str="3.11",
        is_wildcard=True,
        wildcard_precision=2,
        version=None,
    )
    assert spec.contains("3.11.0") is False


def test_specifier_compatible_release_version_none() -> None:
    spec = SimpleSpecifier(
        spec_str="~=3.12",
        operator="~=",
        version_str="3.12",
        is_wildcard=False,
        wildcard_precision=None,
        version=None,
    )
    assert spec._check_compatible_release(SimpleVersion.from_string("3.12")) is False


def test_specifier_eq() -> None:
    assert SimpleSpecifier.from_string(">=3.12") == SimpleSpecifier.from_string(">=3.12")


def test_specifier_eq_not_implemented() -> None:
    result = SimpleSpecifier.from_string(">=3.12").__eq__(">=3.12")  # noqa: PLC2801
    assert result is NotImplemented


def test_specifier_hash() -> None:
    assert hash(SimpleSpecifier.from_string(">=3.12")) == hash(SimpleSpecifier.from_string(">=3.12"))


def test_specifier_str() -> None:
    assert str(SimpleSpecifier.from_string(">=3.12")) == ">=3.12"


def test_specifier_repr() -> None:
    assert repr(SimpleSpecifier.from_string(">=3.12")) == "SimpleSpecifier('>=3.12')"


def test_specifier_version_parse_failure_stores_none() -> None:
    spec = SimpleSpecifier.from_string(">=abc.*")
    assert spec.version is None


def test_specifier_unknown_operator() -> None:
    spec = SimpleSpecifier(
        spec_str="??3.12",
        operator="??",
        version_str="3.12",
        is_wildcard=False,
        wildcard_precision=None,
        version=SimpleVersion.from_string("3.12"),
    )
    assert spec.contains("3.12.0") is False


# --- SimpleSpecifierSet ---


@pytest.mark.parametrize(
    ("spec_str", "version_str", "expected"),
    [
        pytest.param("", "3.12", True, id="empty-always-matches"),
        pytest.param(">=3.12", "3.12.0", True, id="single-match"),
        pytest.param(">=3.12", "3.11.0", False, id="single-no-match"),
        pytest.param(">=3.12,<3.14", "3.12.0", True, id="compound-lower-bound"),
        pytest.param(">=3.12,<3.14", "3.13.0", True, id="compound-middle"),
        pytest.param(">=3.12,<3.14", "3.14.0", False, id="compound-upper-bound"),
        pytest.param(">=3.12,<3.14", "3.11.0", False, id="compound-below"),
    ],
)
def test_specifier_set_contains(spec_str: str, version_str: str, expected: bool) -> None:
    spec_set = SimpleSpecifierSet.from_string(spec_str)
    assert spec_set.contains(version_str) is expected


def test_specifier_set_iter() -> None:
    spec_set = SimpleSpecifierSet.from_string(">=3.12,<3.14")
    specs = list(spec_set)
    assert len(specs) == 2


def test_specifier_set_eq() -> None:
    assert SimpleSpecifierSet.from_string(">=3.12") == SimpleSpecifierSet.from_string(">=3.12")


def test_specifier_set_eq_not_implemented() -> None:
    result = SimpleSpecifierSet.from_string(">=3.12").__eq__(">=3.12")  # noqa: PLC2801
    assert result is NotImplemented


def test_specifier_set_hash() -> None:
    assert hash(SimpleSpecifierSet.from_string(">=3.12")) == hash(SimpleSpecifierSet.from_string(">=3.12"))


def test_specifier_set_str() -> None:
    assert str(SimpleSpecifierSet.from_string(">=3.12")) == ">=3.12"


def test_specifier_set_repr() -> None:
    assert repr(SimpleSpecifierSet.from_string(">=3.12")) == "SimpleSpecifierSet('>=3.12')"


def test_specifier_set_invalid_specifier_skipped() -> None:
    spec_set = SimpleSpecifierSet.from_string(">=3.12, invalid_spec")
    assert len(spec_set.specifiers) == 1


def test_specifier_set_contains_no_specifiers() -> None:
    spec_set = SimpleSpecifierSet.from_string()
    assert spec_set.contains("3.12") is True


def test_specifier_set_empty_item_in_comma_list() -> None:
    spec_set = SimpleSpecifierSet.from_string(">=3.12,,<3.14")
    assert len(spec_set.specifiers) == 2


def test_specifier_compatible_release_major_only() -> None:
    spec = SimpleSpecifier.from_string("~=3")
    assert spec.contains("3.0.0") is True
    assert spec.contains("3.0.5") is True
